// lassoiv.ado -- version 11

capture prog drop lassoiv
program lassoiv

version 11

syntax, panel(varname numeric) time(varname numeric) y(varname numeric) x(varlist numeric min=1) z(varlist numeric min=1) [minnz(real 0.05) w(varlist numeric min=1) xP(varlist numeric min=1)]

// 1. checks for optional scalar parameters
local mnz=.05 // if minnz option is omitted
if strlen("`minnz'")>0 {

local mnz = `minnz'

}

local scalars `mnz'


// 2. checks for optional varlist params: `w' and `xp'.
local params `scalars',"`panel'","`time'","`y'","`x'","`z'","`w'","`xP'"

// now put it all together
mata: t=getparams(`params')
mata: t=correctparams(t)
mata: lasso=setinstruments(t)
mata: writelasso(lasso)

end

mata

// parameters
struct lassoiv_params {
	real colvector panel 
	real colvector time 		   // e.g. "circuit", "year"
	real colvector y 			// dependent variable
	real matrix d, x, z, xP 
	string colvector namez
	real scalar minnz
}

// instruments
struct lassoiv_instruments {
	transmorphic matrix nameznew1, nameznew, table2, table2_1
	transmorphic matrix Original, Select_2, Init, Refined
}
 
// define the program that builds this structure
// -- I guess you might call it a constructor.
function getparams(real scalar mnz, string scalar pvar, string scalar tvar, string scalar yvar, string scalar xvar, string scalar zvar,| string scalar wvar, string scalar xP) {
	struct lassoiv_params scalar t
   // share of non-zero observations (defaults to .05)
   t.minnz=mnz 	
	t.panel=st_data(.,pvar)
	t.time=st_data(.,tvar)
	// dependent variable
	t.y=st_data(.,yvar)   
	// endogenous varlist X; was called d in Matlab.
	t.d=st_data(.,xvar)	
	// set default weights matrix. was called x in Matlab.
	//t.x=(setones(t.panel), setones(t.time), setcircuityear(t.panel,t.time), J(rows(setones(t.time)),1,1))

	// augment it with actual weights, if any.		
	//if (strlen(wvar)!=0) t.x=(st_data(.,wvar), t.x)
	t.x = J(rows(st_data(.,yvar)),1,1)
	if (strlen(wvar)!=0) t.x=(t.x, st_data(.,wvar))
	// Probability Controls
	t.xP = J(rows(st_data(.,yvar)),1,1)
	if (strlen(xP) != 0) t.xP = (t.xP, st_data(.,xP))

	// default z instrument candidates
	t.z=st_data(.,zvar)
	// names of z instrument candidates
	t.namez=tokens(zvar)
	// probability controls
	// t.xP=xP	
		
	return(t)	
}

// drop any rows where there are missing data in any of the matrices:
// t.x, t.z, t.y, t.d, t.xP
struct lassoiv_params scalar correctparams(struct lassoiv_params scalar t) {	
	struct lassoiv_params scalar x
	
	// what changes
	K1=rowmissing(t.x):==0
	K2=rowmissing(t.z):==0
	K3=rowmissing(t.y):==0
	K4=rowmissing(t.d):==0
	K5=rowmissing(t.xP):==0
	K6=rowmissing(t.panel):==0
	K7=rowmissing(t.time):==0
	J=K1:*K2:*K3:*K4:*K5:*K6:*K7

	x.d=select(t.d,J)
	x.z=select(t.z,J)
	x.x=select(t.x,J)
	x.y=select(t.y,J)
	x.xP=select(t.xP,J)
	x.panel=select(t.panel,J)
	x.time=select(t.time,J)
	
	
	// what stays the same
    x.minnz =t.minnz 	
	//x.panel =t.panel
	//x.time  =t.time	
	x.namez =t.namez
		
	return(x)
}


// Subroutines of getparams() constructor above
// 1. build matrices of ones
function setones(string scalar x) {
	myc=st_data(.,x)
	icn=J(rows(myc),max(myc),0)
	for (i=1; i <= rows(myc); i++) { 
   	icn[i,myc[i,1]]=1
   }
	icn=select(icn,colsum(icn):>0)
	icn=select(icn,(0,J(1,cols(icn)-1,1)))
	return(icn)	
}

// 2. build panel-time dummies
// one for the panel, another for the
// time component, to allow for the possibility that they won't always
// be called "circuit" and "year" respectively.
function setcircuityear(string scalar panel, string scalar time) {
	year=st_data(.,time)
	iyearn=setones(time)
	icircuitn=setones(panel)
	jcircuityear=J(rows(icircuitn),cols(icircuitn),0)
	for (i=1;i<=cols(icircuitn);i++)	{
  		jcircuityear[.,i]=year:*icircuitn[.,i]
  	}
  	return(jcircuityear)
}

//getfreq()
//This function returns ordered rows with a column of frequences
real matrix getfreq(real matrix x)
{
     real scalar     i, j, n, ns, f
     real matrix     sortx, res
     
     if (rows(x)==0) return(J(0,cols(x), missingof(x)))
     if (cols(x)==0) return(J(1,0, missingof(x)))

     sortx = sort(x, 1..cols(x))
     ns = 1 
     n = rows(x)
     f = 1
     for (i=2;i<=n;i++) { 
          if (sum(sortx[i-1,]:!=sortx[i,])) ns++
     }
     res = J(ns, cols(x)+1, sortx[1,1])
     res[1,] = (sortx[1,],f)
     for (i=j=2;i<=n;i++) {          
          if (sum(sortx[i-1,] :!= sortx[i,])){
          f = 1
          res[j++,] = (sortx[i,],f)
          }
          else{
          f++
          res[j-1,] = (sortx[i,],f)
          }
     }
     return(res)
}


// cluster_haus()
function cluster_haus(real matrix Score1,real matrix Score2,real matrix H1inv, real matrix H2inv, real matrix group1, real matrix group2)
{
	/*Score1=score_dan
	Score2=score_L
    H1inv=pinv((Md'*zt_dan)*pinv(zt_dan'*zt_dan)*(zt_dan'*Md))*((Md'*zt_dan)*pinv(zt_dan'*zt_dan))
    H2inv=pinv((Md'*zt)*pinv(zt'*zt)*(zt'*Md))*((Md'*zt)*pinv(zt'*zt))
    group1=recode(cy)
    group2=recode(cy)*/
    I=group1:== 1
	J=group2:== 1
    B=(colsum(select(Score1,I)))'*(colsum(select(Score1,I)))
	V1=J(rows(B),cols(B),0)
	B=(colsum(select(Score2,J)))'*(colsum(select(Score2,J)))
	V2=J(rows(B),cols(B),0)
	B=(colsum(select(Score1,I)))'*(colsum(select(Score2,J)))
	CV=J(rows(B),cols(B),0)
	for (ii=1; ii<= max(group1); ii++)
		{
			I=group1:== ii
			J=group2:== ii
			
			V1=V1+(colsum(select(Score1,I)))'*(colsum(select(Score1,I)))
			V2=V2+(colsum(select(Score2,J)))'*(colsum(select(Score2,J)))
			CV=CV+(colsum(select(Score1,I)))'*(colsum(select(Score2,J)))
		}	
	vcluster=(max(group1)/(max(group1)-1))*(H1inv*V1*H1inv' + H2inv*V2*H2inv'- H1inv*CV*H2inv' - H2inv*CV'*H1inv')	
	se=sqrt(diag(vcluster))
	return(vcluster)
}

// findLambdaK
real matrix findLambdaK(real matrix x, real matrix y, real scalar K)
{
   real matrix IND,k
   real scalar L
   numeric scalar iter,swap,direct
   L=4000
   PI=LassoShooting(x,y,L)
   IND=abs(PI):>1.0e-4
   k=colsum(IND)
   
   iter=0
   swap=0
   while ((k != K) & (iter<=1000))
       {
	   		direct=sign(k-K)
       		if (direct>0)
				{
					L=L+5*10^(1-swap)
				}
	   		else
				{	
					L=L-5*10^(1-swap)
				}
	 
	   	PI=LassoShooting(x,y,L)
	   	IND=abs(PI):>1.0e-4
       	k=colsum(IND)
	   	if (direct!= sign(k-K))
				{	
					swap=swap+1
				}
	   
	   	iter=iter+1
         }
   
 /*tmprow=rows(IND)
 tmpcol=cols(IND)+1
 for (i=1; i<=tmprow; i++)
     IND[i,*/

   return ((L\IND))
}

// recode()
real matrix recode(real matrix x)
{
    real scalar tmpmin
    /*x=circuit*/
    tmpmin=min(x)
    tmprows=rows(x)
    x=x-J(tmprows,1,tmpmin)+J(tmprows,1,1)

    table = getfreq(x)
    table=(table,table[.,2]/colsum(table[.,2]))
    K1=rows(table)
    tmp=J(K1,1,1)
    for (i=1; i<=K1; i++) 
        {
        tmp[i,1]=i
        }  
    table=(tmp,table)
    
    for (i=1; i<=max(table[.,2]); i++)
    	{
    	I=x:==i
    	tmp1=J(rows(I),cols(I),0)
    	if (I!=tmp1)
    		{
    			J=table[.,2]:==i
    			/*x[I,1]=table[J,1]*/
    			A=select(table,J)[.,1]
    			for (j=1;j<=rows(x);j++)
    			{    			       
    			      if (I[j]==1)
    			      {
    			         x[j]=A
    			    
    			      }
    			}  
    			    			
    		}
    	}
    
    r=x
    return(r)
}

// LassoShooting()
function LassoShooting(real matrix X,real matrix y,real scalar lambda){

XX=X'*X
Xy=X'*y
n=rows(X)
p=cols(X)
beta=lusolve(XX+lambda*I(p),Xy)
maxIter=10000
verbose=2
optTol=1e-5
zeroThreahold=1e-4
if (verbose==2){
    w_old = beta
    printf("%8s %8s %10s %14s %14s\n","iter","shoots","n(w)","n(step)","f(w)")
    k=1
    wp = beta
    }
m=0
XX2=XX*2
Xy2=Xy*2

while (m < maxIter)
	{
   		beta_old = beta
    	for (j = 1;j<=p;j++)
    	{
       		S0 = colsum(XX2[j,.]*beta) - XX2[j,j]*beta[j] - Xy2[j]
        	if (S0 > lambda)
        	{
            beta[j,1] = (lambda - S0)/XX2[j,j]
            }
        		else {
        				if (S0 < -lambda)	
        				{
            			beta[j,1] = (-lambda - S0)/XX2[j,j]
            			}
        				else {
        						if (abs(S0) <= lambda){
            											beta[j,1] = 0
            											}
            					}
            		}
            }
    
   	m++

	if (verbose==2)
		{
        	printf("%8.0g %8.0g %14.8e %14.8e %14.8e\n",m,m*p,colsum(abs(beta)),colsum(abs(beta-w_old)),colsum((X*beta-y):^2)+lambda*colsum(abs(beta)))
        	w_old = beta
        	k=k+1
        	wp =(wp, beta)
    	}
    
	if (colsum(abs(beta-beta_old))<optTol) break
	}
	
if (verbose)
	{
		printf("Number of iterations: %g\nTotal Shoots: %g\n",m,m*p)
	}
w = beta
return(w)
}

// cluster_se()
function cluster_se(real matrix x,real matrix e,real matrix XpXinv,real matrix group,real scalar k)
{

     /*x=Md
     e=et
     XpXinv=luinv(Md'*Md)
     group=recode(cy)*/
     /*k=kx+1*/
     
     n=rows(e)
     
     /*x=zt
     e=ut
     XpXinv=luinv(zt'*zt)
     group=cyy'
     k=kx+2*/
        if (k==0)
    { 
        k=rows(XpXinv)
    }
    J=group:==1
    B=(select(x,J)'*select(e,J))*(select(x,J)'*select(e,J))'
    V=J(rows(B),cols(B),0)
    for (i=1;i<=max(group);i++)
       {
         I=group:==i
         V=V+(select(x,I)'*select(e,I))*(select(x,I)'*select(e,I))'
        }
    middle=V
    vcluster=((n-1)/(n-k))*(max(group)/(max(group)-1))*XpXinv*V*XpXinv'
    se=sqrt(diagonal(vcluster))
    return(se) 
}

function cluster_var(real matrix x,real matrix e,real matrix XpXinv,real matrix group,real scalar k)
{

     /*x=Md
     e=et
     XpXinv=luinv(Md'*Md)
     group=recode(cy)*/
     /*k=kx+1*/
     
     n=rows(e)
     
     /*x=zt
     e=ut
     XpXinv=luinv(zt'*zt)
     group=cyy'
     k=kx+2*/
        if (k==0)
    { 
        k=rows(XpXinv)
    }
    J=group:==1
    B=(select(x,J)'*select(e,J))*(select(x,J)'*select(e,J))'
    V=J(rows(B),cols(B),0)
    for (i=1;i<=max(group);i++)
       {
         I=group:==i
         V=V+(select(x,I)'*select(e,I))*(select(x,I)'*select(e,I))'
        }
    middle=V
    vcluster=((n-1)/(n-k))*(max(group)/(max(group)-1))*XpXinv*V*XpXinv'
    // se=sqrt(diagonal(vcluster))
    return(vcluster) 
}

// cluster_vcluster()
function cluster_vcluster(real matrix x,real matrix e,real matrix XpXinv,real matrix group,real scalar k)
{
    n=rows(e)
    if (k==0) { 
        k=rows(XpXinv)
    }
    J=group:==1
    B=(select(x,J)'*select(e,J))*(select(x,J)'*select(e,J))'
    V=J(rows(B),cols(B),0)
    for (i=1;i<=max(group);i++)
       {
         I=group:==i
         V=V+(select(x,I)'*select(e,I))*(select(x,I)'*select(e,I))'
        }
    middle=V
    vcluster=((n-1)/(n-k))*(max(group)/(max(group)-1))*XpXinv*V*XpXinv'
    se=sqrt(diagonal(vcluster))
    return(vcluster) 
}

// setinstruments() function encapsulates loose Mata code, 
// minus the part that writes Stata files. That part will be handled by
// a separate void function.
struct lassoiv_instruments scalar setinstruments(struct lassoiv_params scalar t) {
	
	panel=t.panel
	time=t.time
	y=t.y
	x=t.x
	d=t.d
	z=t.z	
	xP=t.xP
	namez=t.namez'
   minnz=t.minnz
   
	
	struct lassoiv_instruments scalar dlc_lasso

	I=mean(z):>=minnz
	z=select(z,I)
	namez=select(namez,I')
	
	I=mean(xP):>=minnz
	xP=select(xP,I)

	if (cols(xP) > 1) {
	xP = xP[|1,2\.,.|]

	x=(x,xP)
	}

	
	xxinv = luinv(x'*x)
	
	My = y - x*xxinv*(x'*y)
	Md = d - x*xxinv*(x'*d)
	Mz = z - x*xxinv*(x'*z)
	
    
	std_Mz=sqrt(diagonal(variance(Mz)))'
	I=std_Mz:>1e-6
	J=colmissing(std_Mz):==0	
    
    //I=I:*J
	Mz=select(Mz,I)
	namez=select(namez,I')

	n=rows(Mz)
	std_Mz=sqrt(diagonal(variance(Mz)))'

	sd=sqrt(diagonal(variance(Md)))'

	Sz_app=J(rows(Mz),1,1)*std_Mz
	Md=Md/sd
	Mz=Mz:/Sz_app

	K=cols(Mz)
	A=findLambdaK(Mz,Md,2)
	
	L=A[1,.]
	IND=select(A,diagonal(I(rows(A)))-I(rows(A))[.,1])

	colsum(IND)
	v = Md - select(Mz,IND')*luinv(select(Mz,IND')'*select(Mz,IND'))*(select(Mz,IND')'*Md)

	// year=st_data(.,time)
	year = time
	// circuit=st_data(.,panel)
	circuit = panel
	
	recodecircuit = recode(circuit)
	recodeyear = recode(year)
	
	cy = (recodecircuit-J(rows(recodecircuit),cols(recodecircuit),1)):*max(recodeyear)+recodeyear
	
	Dt=J(rows(cy),max(cy),0)
	for (i=1; i <= rows(cy); i++)
	   { 
	   Dt[i,cy[i,1]]=1
	   }
	  
	St=Mz:*(v*J(1,cols(Mz),1))
	Ups = sqrt(colsum((Dt'*St):^2)/n)
	

	c = 1.1
	lambda=c*2*sqrt(2*n*(log(2*(cols(Mz)))))
	PIpA = LassoShooting(Mz:/(J(n,1,1)*Ups), Md , lambda)
	   
    INDpA = abs(PIpA):> 1.0e-4      

	colsum(INDpA)
	
	v = Md - select(Mz,INDpA')*luinv(select(Mz,INDpA')'*select(Mz,INDpA'))*(select(Mz,INDpA')'*Md)

	St = Mz:*(v*J(1,cols(Mz),1))
	UpsR = sqrt(colsum((Dt'*St):^2)/n)
	PIpL = LassoShooting(Mz:/(J(n,1,1)*UpsR), Md , lambda)

    INDpL = abs(PIpL):> 1.0e-4
   
	colsum(INDpL)
	
	Md = Md*sd
	Mz = Mz:*Sz_app
	
	kx = cols(x)
	
	recodecy = recode(cy)
	G = max(recodecy)

	OLS = luinv(Md'*Md)*(Md'*My)
	et = My-Md*OLS
	sOLS = cluster_se(Md,et,luinv(Md'*Md),recodecy,kx+1)
	
	// cyy  = (1..rows(x))
	// sOLS = cluster_se(Md,et,luinv(Md'*Md),cyy',kx+1)
	
	
	IND_dan = J(cols(Mz),1,0)
	IND_dan[1] = 1
	IND_dan[2] = 1
	// IND_dan = IND_dan:>0
	
	/***Calculate first stage***/
	zt = select(Mz,IND_dan')
	FS_dan = luinv(zt'*zt)*(zt'*Md)
	ut = Md-zt*FS_dan
	sFS_dan= cluster_se(zt,ut,luinv(zt'*zt),recodecy,kx+2)
	VFS_dan= cluster_var(zt,ut,luinv(zt'*zt),recodecy,kx+2)
	FS_F_dan = (FS_dan'*luinv(VFS_dan)*FS_dan)*((G-2)/(2*(G-1)))
	FS_p_dan = 1-F(2,G-2,FS_F_dan)

	/****Calculate 2sls estimate***/
	TSLS_dan = luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)*(zt'*My))
	et = My-Md*TSLS_dan
	sTSLS_dan = cluster_se(zt,et,luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)),recode(cy),kx+1)
		
	/****
	% Calculate results using LASSO choosing 2 instruments
	% Calculate first stage***/	
	zt = select(Mz,IND')
	FS_2 = luinv(zt'*zt)*(zt'*Md)
	ut = Md-zt*FS_2
	sFS_2 = cluster_se(zt,ut,luinv(zt'*zt),recodecy,kx+2)
	VFS_2 =cluster_var(zt,ut,luinv(zt'*zt),recodecy,kx+2)
	FS_F_2 = (FS_2'*luinv(VFS_2)*FS_2)*((G-2)/(2*(G-1)))
	FS_p_2 = 1-F(2,G-2,FS_F_2)

	/***Calculate 2sls estimate***/
	TSLS_2 = luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)*(zt'*My))
	et = My-Md*TSLS_2
	sTSLS_2 = cluster_se(zt,et,luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)),recodecy,kx+1)
	
	/***% Calculate results using LASSO A penalty
	% Calculate first stage****/
	zt = select(Mz,INDpA')
	qA = colsum(INDpA)
	FS_A = luinv(zt'*zt)*(zt'*Md)
	ut = Md-zt*FS_A
	sFS_A= cluster_se(zt,ut,luinv(zt'*zt),recodecy,kx+qA)
	VFS_A= cluster_var(zt,ut,luinv(zt'*zt),recodecy,kx+qA)
	FS_F_A = (FS_A'*luinv(VFS_A)*FS_A)*((G-qA)/(qA*(G-1)))
	if (FS_F_A <= 0){
	    FS_F_A = .
	}
	FS_p_A = 1-F(qA,G-qA,FS_F_A)

	/***Calculate 2sls estimate***/	
	TSLS_A = luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)*(zt'*My))
	et = My-Md*TSLS_A
	sTSLS_A = cluster_se(zt,et,luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)),recodecy,kx+1)
	
	/***% Calculate results using LASSO L penalty
	% Calculate first stage****/
	zt = select(Mz,INDpL')
	qL = colsum(INDpL)
	FS_L = luinv(zt'*zt)*(zt'*Md)
	ut = Md-zt*FS_L
	sFS_L = cluster_se(zt,ut,luinv(zt'*zt),recodecy,kx+qL)
	VFS_L = cluster_var(zt,ut,luinv(zt'*zt),recodecy,kx+qL)
	FS_F_L = (FS_L'*luinv(VFS_L)*FS_L)*((G-qL)/(qL*(G-1)))
	if (FS_F_L <= 0){
	    FS_F_L = .
	}
	FS_p_L = 1-F(qL,G-qL,FS_F_L)
	
	/***Calculate 2sls estimate***/
	TSLS_L = luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)*(zt'*My))
	et = My-Md*TSLS_L
	sTSLS_L = cluster_se(zt,et,luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)),recodecy,kx+1)

	zt_dan = select(Mz,IND_dan')
	et_dan = My-Md*TSLS_dan
	score_dan = zt_dan:*(et_dan*J(1,2,1))
	score_L = zt:*(et*J(1,cols(zt),1))
	se_dif = cluster_haus(score_dan,score_L,luinv((Md'*zt_dan)*luinv(zt_dan'*zt_dan)*(zt_dan'*Md))*((Md'*zt_dan)*luinv(zt_dan'*zt_dan)),luinv((Md'*zt)*luinv(zt'*zt)*(zt'*Md))*((Md'*zt)*luinv(zt'*zt)),recodecy,recodecy)
	t_dif = (TSLS_dan-TSLS_L)/se_dif
	
	// now fill in dlc_lasso elements
	dlc_lasso.nameznew1=(IND,INDpA,INDpL)
	dlc_lasso.nameznew =namez
	dlc_lasso.table2   =(OLS,TSLS_dan,TSLS_2,TSLS_A,TSLS_L\sOLS,sTSLS_dan,sTSLS_2,sTSLS_A,sTSLS_L\.,FS_F_dan,FS_F_2,FS_F_A,FS_F_L\.,FS_p_dan,FS_p_2,FS_p_A,FS_p_L\.,2,2,qA,qL\G,G,G,G,G)
	dlc_lasso.table2_1 =("bhat"\"se"\"FS_F"\"FS_p"\"q"\"G")
	
	// instruments
	dlc_lasso.Original=select(namez,IND_dan)
	dlc_lasso.Select_2=select(namez,IND)
	dlc_lasso.Init=select(namez,INDpA)
	dlc_lasso.Refined=select(namez,INDpL)
	
	return(dlc_lasso)
}

// writelasso() encapsulates the part of loose Mata code that writes Stata files.
void writelasso(struct lassoiv_instruments scalar r) {
	// form1
	stata("preserve")
	stata("clear")
	YY=r.nameznew1
	index=st_addvar("long",("Select_2","Init","Refined"))
	st_addobs(rows(YY))
	st_store((1,rows(YY)),index,YY)
	stata("save dlc_nameznew1,replace")
	stata("clear")
	YY=r.nameznew
	index=st_addvar("str50","varname")
	st_addobs(rows(YY))
	st_sstore((1,rows(YY)),index,YY)
	stata("save dlc_nameznew,replace")
	stata("merge 1:1 _n using dlc_nameznew1")
	stata("drop _merge")
	stata("save dlc_test1_ols1,replace")

	// form2
	stata("clear")
	YY=r.table2
	index=st_addvar("double",("OLS","Original","Select_2","Init","Refined"))
	st_addobs(rows(YY))
	st_store((1,rows(YY)),index,YY)
	stata("save dlc_table2,replace")
	stata("clear")
	YY=r.table2_1
	index=st_addvar("str50","Row")
	st_addobs(rows(YY))
	st_sstore((1,rows(YY)),index,YY)
	stata("save dlc_table2_1,replace")
	stata("merge 1:1 _n using dlc_table2")
	stata("drop _merge")
	stata("save dlc_test1_ols2,replace")
	
	// instruments
	stata("clear")
	YY=r.Original
	index=st_addvar("str50","Original")
	st_addobs(rows(YY))
	st_sstore((1,rows(YY)),index,YY)
	stata("save Original,replace")
	
	stata("clear")
	YY=r.Select_2
	index=st_addvar("str50","Select_2")
	st_addobs(rows(YY))
	st_sstore((1,rows(YY)),index,YY)
	stata("save Select_2,replace")
	
	stata("clear")
	YY=r.Init
	index=st_addvar("str50","Init")
	st_addobs(rows(YY))
	st_sstore((1,rows(YY)),index,YY)
	stata("save Init,replace")
	
	stata("clear")
	YY=r.Refined
	index=st_addvar("str50","Refined")
	st_addobs(rows(YY))
	st_sstore((1,rows(YY)),index,YY)
	stata("save Refined,replace")
	
	stata("clear")
	stata("use Original")
	stata("merge 1:1 _n using Select_2")
	stata("drop _merge")
	stata("merge 1:1 _n using Init")
	stata("drop _merge")
	stata("merge 1:1 _n using Refined")
	stata("drop _merge")

	stata("save instruments,replace")
	stata("restore")	
}

end

